// ====================================================================
//  logger.cxx Remote Log Interface for fldigi
//
// Copyright W1HKJ, Dave Freese 2006
//
// This library is free software; you can RGBredistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
//
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
// Please report all bugs and problems to "w1hkj@w1hkj.com".
//
// ====================================================================

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <errno.h>

#include "logger.h"
#include "main.h"
#include "version.h"
#include "modem.h"

#include <FL/fl_ask.H>

//---------------------------------------------------------------------

static msgtype msgbuf;
static string log_msg;
static string errmsg;
static char strFreqMhz[20];
char   LOG_MSEPARATOR[2] = {1,0};


//---------------------------------------------------------------------

int submit_log(void)
{
	char date[32], logtime[32];
	int msqid, len;
	struct tm *tm;
	time_t t;

	time(&t);
        tm = gmtime(&t);
		strftime(date, sizeof(date), "%d %b %Y", tm);
		strftime(logtime, sizeof(logtime), "%H%M", tm);

	string mode = mode_names[active_modem->get_mode()];
	
	sprintf(strFreqMhz,"%-12f", wf->dFreq()/1.0e6);
	
	Fl::lock();
	log_msg = "";
	log_msg = log_msg + "program:"	+ "fldigi v " + FLDIGI_VERSION + LOG_MSEPARATOR;
	log_msg = log_msg + "version:"	+ LOG_MVERSION			+ LOG_MSEPARATOR;
	log_msg = log_msg + "date:"		+ date					+ LOG_MSEPARATOR;
	log_msg = log_msg + "time:"		+ inpTime->value()		+ LOG_MSEPARATOR;
	log_msg = log_msg + "endtime:"	+ logtime				+ LOG_MSEPARATOR;
	log_msg = log_msg + "call:"		+ inpCall->value()		+ LOG_MSEPARATOR;
	log_msg = log_msg + "mhz:"		+ strFreqMhz			+ LOG_MSEPARATOR;
	log_msg = log_msg + "mode:"		+ mode					+ LOG_MSEPARATOR;
	log_msg = log_msg + "tx:"		+ inpRstOut->value()	+ LOG_MSEPARATOR;
	log_msg = log_msg + "rx:"		+ inpRstIn->value() 	+ LOG_MSEPARATOR;
	log_msg = log_msg + "name:"		+ inpName->value()		+ LOG_MSEPARATOR;
	log_msg = log_msg + "qth:"		+ inpQth->value()		+ LOG_MSEPARATOR;
	log_msg = log_msg + "locator:"	+ inpLoc->value()		+ LOG_MSEPARATOR;
	log_msg = log_msg + "notes:"	+ inpNotes->value()		+ LOG_MSEPARATOR;
	Fl::unlock();

	strcpy(msgbuf.mtext, log_msg.c_str());
	
	if ((msqid = msgget(LOG_MKEY, 0666 | IPC_CREAT)) == -1) {
		errmsg = "msgget: ";
		errmsg.append(strerror(errno));
		fl_message(errmsg.c_str());
		return -1;
	}
	msgbuf.mtype = LOG_MTYPE;

// allow for the NUL
	len = strlen(msgbuf.mtext) + 1;
	if (msgsnd(msqid, &msgbuf, len, IPC_NOWAIT) < 0) {
		errmsg = "msgsnd: ";
		fl_message(errmsg.c_str());
		return -1;
	}
	return 0;
}

//---------------------------------------------------------------------

