#include "macros.h"

#include "main.h"

#include "fl_digi.h"
#include "configuration.h"
#include "Config.h"
#include "version.h"
#include "logger.h"
#include "newinstall.h"

#include <FL/Fl.H>
#include "File_Selector.h"

#include <string>
#include <ctime>
#include <iostream>
#include <fstream>

MACROTEXT macros;
CONTESTCNTR contest_count;

int MACROTEXT::loadMacros(string filename)
{
	string mLine;
	string mName;
	string mDef;
	bool   inMacro = false;
	int    mNumber = 0;
	unsigned long int	   crlf; // 64 bit cpu's
	char   szTemp[10];
	char   szLine[255];
	
	ifstream *mFile;

	mFile = new ifstream(filename.c_str());
	
	if (!mFile->is_open()) {
		createDotFldigi();
		mFile = new ifstream(filename.c_str());
		if (!mFile->is_open())
			return -1;
	}
	
	mFile->getline(szLine, 255);
	mLine = szLine;
	if (mLine.find("//fldigi macro definition file") != 0) {
		mFile->close();
		return -1;
	}
// clear all of the macros
	for (int i = 0; i < 20; i++) {
		sprintf(szTemp,"%d", i+1);
		name[i] = "Macro ";
		name[i] = name[i] + szTemp;
		text[i] = "";
	}
	inMacro = false;
	while (!mFile->eof()) {
		mFile->getline(szLine,255);
		mLine = szLine;
		if (mLine.find("//") == 0) // skip over all comment lines
			continue;
		if (mLine.find("/$") == 0) {
			int idx = mLine.find_first_not_of("0123456789", 3);
			sscanf((mLine.substr(3, idx - 3)).c_str(), "%d", &mNumber);
			if (mNumber < 0 || mNumber > 19)
				break;
			name[mNumber] = mLine.substr(idx+1, 8);
			if (mNumber < 10) {
				Fl::lock();
				btnMacro[mNumber]->label( (macros.name[mNumber]).c_str());
				Fl::unlock();
			}
			continue;
		}
		while ((crlf = mLine.find("\\n")) != string::npos) {
			mLine.erase(crlf, 2);
			mLine.append("\n");
		}
		text[mNumber] = text[mNumber] + mLine;
	}
	mFile->close();
	return 0;
}

void MACROTEXT::loadDefault()
{
	string Filename = HomeDir;
	Filename.append("macros.mdf");
	loadMacros(Filename);
}

void MACROTEXT::openMacroFile()
{
	string deffilename = HomeDir;
	deffilename.append("/macros.mdf");
    char *p = File_Select("Open macro file", "*.mdf", deffilename.c_str(), 0);
    if (p)
		loadMacros(p);
}

size_t mystrftime( char *s, size_t max, const char *fmt, const struct tm *tm) {
	return strftime(s, max, fmt, tm);
}

string MACROTEXT::expandMacro(int n)
{
	unsigned long int idx; // 64 bit cpu's
	string dt;
	string progname;
	char szDt[80];
	time_t tmptr;
	tm sTime;
	int  number;
	int  contestval;
	
	expanded = text[n];
	time (&tmptr);
	if (expanded.find("^t") == 0)
		expanded.erase(0,2);
	while ((idx = expanded.find("^")) != string::npos) {
		if (expanded[idx+1] == 'r')
			break;
		switch (expanded[idx+1]) {
			case '^' : expanded.replace (idx, 1, ""); break;
			case '>' : 
				sscanf(expanded.substr(idx+2).c_str(), "%d", &number);
				unsigned long int idx2;
				idx2 = expanded.find(" ",idx);
				if (idx2 == string::npos)
					idx2 = expanded.find("\n", idx);
				expanded.replace (idx, idx2 - idx + 1, "");
				progdefaults.timeout = number;
				progdefaults.macronumber = n;
				progdefaults.useTimer = true;
				break;
			case 'c' : expanded.replace( idx, 2, inpMyCallsign->value() ); break;
			case 'd' :
				localtime_r(&tmptr, &sTime);
				mystrftime(szDt, 79, "%x %H:%M %Z", &sTime);
				expanded.replace( idx, 2, szDt);
				break;
			case 'D' :
				gmtime_r(&tmptr, &sTime);
				mystrftime(szDt, 79, "%x %H:%M %Z", &sTime);
				expanded.replace( idx, 2, szDt);
				break;
			case 'f' : expanded.replace( idx, 2, inpFreq->value() ); break;
			case 'h' :
				localtime_r(&tmptr, &sTime);
				asctime_r(&sTime, szDt);
				dt = szDt;
				dt.erase(dt.end() -1); // remove the \n from the asctime string
				expanded.replace( idx, 2, dt); 
				break;
			case 'i' :
				progdefaults.macroid = true;
				expanded.replace( idx, 2, "");
				break;
			case 'l' : expanded.replace( idx, 2, inpMyLocator->value() ); break;
			case 'n' : expanded.replace( idx, 2, inpMyName->value() ); break;
			case 'm' : expanded.replace( idx, 2, active_modem->get_mode_name()); break;
			case 'q' : expanded.replace( idx, 2, inpMyQth->value() ); break;
			case 's' : expanded.replace( idx, 2, inpRstIn->value() ); break;
			case 'v' : 
				progname = "Fldigi ";
				progname.append(FLDIGI_VERSION);
				expanded.replace( idx, 2, progname );
				break;
			case 'z' :
			case 'Z' :
				gmtime_r(&tmptr, &sTime);
				asctime_r(&sTime, szDt);
				dt = szDt;
				dt.erase(dt.end() -1); // remove the \n from the asctime string
				expanded.replace( idx, 2, dt); 
				break;
			case 'C' : expanded.replace( idx, 2, inpCall->value() ); break;
			case 'L' : expanded.replace( idx, 2, inpLoc->value() ); break;
			case 'N' : expanded.replace( idx, 2, inpName->value() ); break;
			case 'Q' : expanded.replace( idx, 2, inpQth->value() ); break;
			case 'S' : expanded.replace( idx, 2, inpRstOut->value() ); break;
			case '?' :
				contestval = contest_count.count + progdefaults.ContestStart;
				contest_count.Format(progdefaults.ContestDigits, progdefaults.UseLeadingZeros);
				sprintf(contest_count.szCount, contest_count.fmt.c_str(), contestval);
				expanded.replace (idx, 2, contest_count.szCount);
				break;
			case '+' :
				contest_count.count++;
				contestval = contest_count.count + progdefaults.ContestStart;
				expanded.replace (idx, 2, "");
				sprintf(contest_count.szCount, contest_count.fmt.c_str(), contestval);
				sprintf(contest_count.szDisp,"Next Nbr: %s", contest_count.szCount);
				put_status(contest_count.szDisp);
				break;
			case '-' :
				contest_count.count--;
				contestval = contest_count.count + progdefaults.ContestStart;
				expanded.replace (idx, 2, "");
				sprintf(contest_count.szCount, contest_count.fmt.c_str(), contestval);
				sprintf(contest_count.szDisp,"Next Nbr: %s", contest_count.szCount);
				put_status(contest_count.szDisp);
				break;
			case '#' :
				submit_log();
				expanded.replace(idx,2,"");
				clearQSO();
				break;
			default  : expanded.replace(idx,2,""); break;
		}
	}
	return expanded;
}

void MACROTEXT::execute(int n) 
{
	if ( ((text[n]).find("^t")) == 0) {
		active_modem->set_stopflag(false);
		fl_lock(&trx_mutex);
		trx_state = STATE_TX;
		fl_unlock(&trx_mutex);
		wf->set_XmtRcvBtn(true);
	}
	TransmitText->add( (expandMacro(n)).c_str() );
}

