#include "macros.h"

#include "main.h"

#include "fl_digi.h"
#include "configuration.h"
#include "Config.h"
#include "version.h"
#include "logger.h"

#include <FL/Fl.H>
#include "File_Selector.h"

#include <string>
#include <ctime>
#include <iostream>
#include <fstream>
#include <sys/stat.h>


string mtext = 
"//fldigi macro definition file\n\
// This file defines the macro structure(s) for the digital modem program, fldigi\n\
// It also serves as a basis for any macros that are written by the user\n\
//\n\
// The top line of this file should always be the first line in every macro definition file (.mdf)\n\
// for the fldigi program to recognize it as such.\n\
//\n\
// Macros are short text statements that contain imbedded references to text data used by the\n\
// program fldigi.  The imbedded references are always prefaced by the carat(^) symbol and consist\n\
// of single letter codes.  All references to the local system are in lower case and to the remote\n\
// system or global values are in upper case.\n\
//\n\
//\n\
// ^c  my call                        ^C  remote call\n\
// ^d  local date time Zone\n\
//     format : %x %H:%M %Z\n\
//     where %x is preferred short form date ie: MM/DD/YY or DD/MM/YYYY etc\n\
//           %H is hour with leading 0\n\
//           %M is minute with leading 0\n\
//           %Z is abbreviated time zone ie: EDT or GMT\n\
// ^D  GMT date time Zone\n\
//     format : %x %H:%M %Z\n\
// ^f  my frequency\n\
// ^h  my local time\n\
//     format : Day Mon day# HH:MM:SS YYYY\n\
// ^i  send Mode Idenfier - waterfall script\n\
// ^l  my locator                     ^L  remote locator\n\
// ^m  my mode\n\
// ^n  my name                        ^N  remote name\n\
// ^q  my qth                         ^Q  remote qth\n\
// ^r  return to receive\n\
// ^s  my RST                         ^S  remote RST\n\
// ^t  start transmit\n\
// ^v  Fldigi + version\n\
// ^z  my gmt time\n\
// ^Z  my gmt time\n\
//     format : Day Mon day# HH:MM:SS YYYY\n\
// Reference definitions:\n\
//\n\
// ^+  increment the contest counter - no substitution\n\
// ^-  decrement the contest counter - no substitution\n\
// ^?  substitute the contest counter - no change in value\n\
//\n\
// ^#  submit QSO data to logbook program & clear the QSO data fields\n\
//     Not constrained to a particular position in the macro.\n\
//     Action takes place when macro is expanded, so effect is seen immediately\n\
//     after executing the macro which contains this macro reference.\n\
//\n\
// ^>NNN autorepeat this macro after NNN seconds\n\
// NNN must be terminated by either a space ' ' or a linefeed\n\
// it can appear anywhere in the macro defintion.\n\
// see macro # 19 for an example of an auto-cq repeating macro\n\
//\n\
//\n\
// Local references are specified during the program configuration and can be changed during \n\
// program operation.\n\
//\n\
// Remote references are all part of the qso log field definitions and are routinely changed \n\
// from contact to contact.\n\
//\n\
// Global references are to things like Greenwich Mean Time\n\
//\n\
// Each new macro begins with a macro specifier line as: /$ nn MACRONAME\n\
//   where 'nn' specifies the macro number.  Macros numbered 0..9 refer to function key 1 to 10\n\
//   respectively.  Macros numbered 10..19 refer to \"alt\" function keys 1 to 10.  Undefined Macros\n\
//   will also contain an empty character string and thus produce no output when invoked.\n\
// MACRONAME can be as long as you want, however only the first 8 characters will be displayed on\n\
//   the associated function key button.\n\
//\n\
// You can put the macro definition on multiple lines.  These lines will be concatenated\n\
// into a single line unless you put the new-line pair \"\\n\" at the end of the line\n\
// that you want to terminate a line during transmission.\n\
//\n\
// Macro definitions do not need to be in numerical order.\n\
// Macro definitions may be skipped or you can truncate the file to less than\n\
// the full macro set of 20.\n\
//\n\
// I recommend using a copy of this file suitably modified for all of your macro\n\
// definition files.  Just fill in the appropriate fields and delete those you do not\n\
// need for that macro set.\n\
//\n\
//\n\
// Let the Macros begin!\n\
//\n\
// MACRO 1 --- count starts at 0\n\
/$ 0 f1 CQ\n\
^t\\n\n\
CQ CQ CQ de ^c ^c ^c\\n\n\
CQ CQ CQ de ^c ^c ^c pse k\\n\n\
^r\n\
// YOU CAN PUT COMMENT LINES ANYWHERE YOU WANT IN THE MACRO DEFINITIONS\n\
//\n\
// MACRO 2\n\
/$ 1 f2 ANS\n\
^t\\n\n\
^C ^C de ^c ^c ^c kn\\n\n\
^r\n\
//\n\
// MACRO 3\n\
/$ 2 f3 QSO\n\
^t\\n\n\
^C de ^c \n\
//\n\
// MACRO 4\n\
/$ 3 f4 KN\n\
btu ^N ^C de ^c k\\n\n\
^r\n\
//\n\
// MACRO 5\n\
/$ 4 f5 SK\n\
\\n\n\
tnx fer QSO ^N, 73, God bless.\\n\n\
^D ^C de ^c sk\\n\n\
^r\n\
//\n\
// MACRO 6\n\
/$ 5 f6 Me\n\
 my name is ^n ^n \n\
//\n\
// MACRO 7\n\
/$ 6 f7 QTH\n\
 my QTH is ^q, loc: ^l \n\
//\n\
// MACRO 8\n\
/$ 7 f8 Brag    ANTHING PAST 8 CHARACTERS IS DISCARDED, SO THIS CAN ALSO BE A COMMENT\n\
\\n\n\
(( ^c, ^n Info ))\\n\n\
Age:   \\n\n\
Rig:   \\n\n\
Pwr:   \\n\n\
Ant:   \\n\n\
OS:    Linux\\n\n\
Soft:  ^v\\n\n\
Web:   \\n\n\
Email: \\n\n\
//\n\
// MACRO 9\n\
/$ 8 f9 Test\n\
^t\\n\n\
vvvv vvvv vvvv de ^c ^c k^r\\n\n\
//\n\
// MACRO 10\n\
/$ 9 f10 Lcall\n\
^t^C ^C ^C ^C de ^c ^c kn^r\\n\n\
//\n\
// MACRO 11\n\
/$ 10 Ttest\
^h -- ^z\\n\n\
^d -- ^D\\n\n\
^v\\n\n\
//\n\
// MACRO 12\n\
/$ 11 C Ans     answer contest CQ call\n\
^tde ^c ^c^r\\n\n\
//\n\
// MACRO 13\n\
/$ 12 C Repeat  repeat exchange\n\
^t^-^? ^?^+ QSL DE ^c K\\n\n\
^r\n\
//\n\
// MACRO 14\n\
/$ 13 C Report  make report\n\
^t^C RR UR NBR ^? ^?^+ TU DE ^c K\\n\n\
^r\n\
//\n\
// MACRO 15     increment the contest counter\n\
/$ 14 C Incr\n\
^+\n\
//\n\
// MACRO 16     decrement the contest counter\n\
/$ 15 C Decr\n\
^-\n\
//\n\
// MACRO 17\n\
/$ 16 CW-CQ\n\
^tCQ CQ CQ DE ^c ^c ^c  CQ CQ CQ DE ^c K^r\\n\n\
//\n\
// MACRO 18\n\
/$ 17 Macro 18\n\
//\n\
// MACRO 19\n\
/$ 18 AUTO-CQ\n\
^t\\n\n\
CQ CQ CQ de ^c ^c ^c\\n\n\
CQ CQ CQ de ^c ^c ^c k^>15 ^r\n\
//\n\
// MACRO 20\n\
/$ 19 CQ-ID\n\
^t^i\\n\n\
CQ CQ CQ de ^c ^c ^c\\n\n\
CQ CQ CQ de ^c ^c ^c pse k\\n\n\
^r\n\
\
";

void createDotFldigi()
{
	string Filename = HomeDir;
	mkdir(Filename.c_str(), O_CREAT | S_IRUSR|S_IWUSR|S_IXUSR);
	Filename.append("macros.mdf");
	ofstream mfile(Filename.c_str());
	mfile << mtext;
	mfile.close();
}
