// ----------------------------------------------------------------------------
//
//    ptt.cxx --  PTT control
//
// Copyright (C) 2006
//		Dave Freese, W1HKJ
//
// This file is part of fldigi.  Adapted from code contained in gmfsk source code 
// distribution.
//  gmfsk Copyright (C) 2001, 2002, 2003
//  Tomi Manninen (oh2bns@sral.fi)
//  Copyright (C) 2004
//  Lawrence Glaister (ve7it@shaw.ca)
//
// fldigi is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// fldigi is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with fldigi; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
// ----------------------------------------------------------------------------

#include "ptt.h"
#include "configuration.h"
#include "rigMEM.h"
#include "rigio.h"
#ifndef NOHAMLIB
	#include "hamlib.h"
#endif

#include <FL/fl_ask.H>


// dev may be one of
// "none" - 0
// "hamlib" - 1
// "memmap" - 2
// "rigcat" - 3
// "/dev/ttys0" - 5
// "/dev/ttys1" - 5

PTT::PTT(int dev, int mode, bool inverted)
{
	pttfd = -1;
	reset_(dev, mode, inverted);
}

PTT::~PTT()
{
	if (pttdev < 3)
		return;

	if (pttfd != -1) {
		tcsetattr (pttfd, TCSANOW, &oldtio);
		close(pttfd);
		pttfd = -1;
	}
}

void PTT::reset_(int dev, int mode, bool inverted)
{
	pttdev = dev;
	pttinv = inverted;
	pttmode = mode;

	if (pttdev < 4) {
		set(0);
		return;
	}

// four types of dev/tty control
// 0 - RTS only
// 1 - DTR only
// 2 - RTS & DTR
// 3 - RTS control, DTR always on	
	switch (pttmode) {
	case 0:
		pttarg = TIOCM_RTS;
		break;
	case 1:
		pttarg = TIOCM_DTR;
		break;
	case 2:
		pttarg = TIOCM_RTS | TIOCM_DTR;
		break;
	case 3:
		pttarg = TIOCM_RTS;
		break;
	default:
		pttarg = TIOCM_RTS | TIOCM_DTR;
	}
	pttdevName = progdefaults.PTTdev;
	
	openptt();
	if (pttfd == -1) {
		string msg = "Cannot open ";
		msg = msg + pttdevName;
		fl_message( msg.c_str() );
		return;
	}
	set(0);
}


void PTT::reset(int dev, int mode, bool inverted)
{
	if (pttfd != -1) {
		tcsetattr (pttfd, TCSANOW, &oldtio);
		close(pttfd);
		pttfd = -1;
	}
	reset_(dev, mode, inverted);
}

void PTT::set(bool ptt)
{
	if (pttdev == 0) return;

	if (active_modem == cw_modem && 
			((progdefaults.useCWkeylineRTS == true) || 
			 (progdefaults.useCWkeylineDTR == true) ) )
		return;

// Hamlib ptt
#ifndef NOHAMLIB
	if (pttdev == 1) {
		hamlib_set_ptt(ptt);
		return;
	}
#endif
// Memory mapped i/o
	if (pttdev == 2) { 
		setrigMEM_PTT (ptt);
		return;
	}
	if (pttdev == 3){
		rigCAT_set_ptt (ptt);
		return;
	}
	if (pttfd == -1)
		return;
	
	if (pttinv)
		ptt = !ptt;

	tmparg = pttarg;
	
	if (pttmode == 3) {
		ioctl(pttfd, TIOCMGET, &tmparg);
		tmparg |= TIOCM_DTR; 		// set the DTR bit
		if (ptt) 				// set or clear the RTS bit
		  tmparg |= TIOCM_RTS;
		else
			tmparg &= ~TIOCM_RTS;
		ioctl(pttfd, TIOCMSET, &tmparg);
	} else
		ioctl(pttfd, ptt ? TIOCMBIS : TIOCMBIC, &tmparg);
}


void PTT::openptt()
{
	unsigned int arg = pttarg;
	
	if ((pttfd = open(pttdevName.c_str(), O_RDWR, 0)) < 0)
		return;

	tcgetattr (pttfd, &oldtio);

	if (ioctl(pttfd, TIOCMBIC, &arg))
		pttfd = -1;

	return;
}


